@@ -1,33 +1,14 @@
-from pathlib import Path
-from typing import Any, List, Optional, Tuple
+from typing import Any, ClassVar, Optional, Tuple
 
 from datamodel_code_generator.imports import IMPORT_ANY, IMPORT_ENUM, Import
 from datamodel_code_generator.model import DataModel, DataModelFieldBase
-from datamodel_code_generator.reference import Reference
 from datamodel_code_generator.types import DataType, Types
 
 
 class Enum(DataModel):
-    TEMPLATE_FILE_PATH = 'Enum.jinja2'
-    BASE_CLASS = 'enum.Enum'
-
-    def __init__(
-        self,
-        *,
-        reference: Reference,
-        fields: List[DataModelFieldBase],
-        decorators: Optional[List[str]] = None,
-        path: Optional[Path] = None,
-        description: Optional[str] = None,
-    ):
-        super().__init__(
-            fields=fields,
-            reference=reference,
-            decorators=decorators,
-            path=path,
-            description=description,
-        )
-        self._additional_imports.append(IMPORT_ENUM)
+    TEMPLATE_FILE_PATH: ClassVar[str] = 'Enum.jinja2'
+    BASE_CLASS: ClassVar[str] = 'enum.Enum'
+    DEFAULT_IMPORTS: ClassVar[Tuple[Import, ...]] = (IMPORT_ENUM,)
 
     @classmethod
     def get_data_type(cls, types: Types, **kwargs: Any) -> DataType:
