@@ -27,7 +27,12 @@ from datamodel_code_generator import Protocol, runtime_checkable
 from datamodel_code_generator.format import CodeFormatter, PythonVersion
 from datamodel_code_generator.imports import IMPORT_ANNOTATIONS, Import, Imports
 from datamodel_code_generator.model import pydantic as pydantic_model
-from datamodel_code_generator.model.base import ALL_MODEL, DataModel, DataModelFieldBase
+from datamodel_code_generator.model.base import (
+    ALL_MODEL,
+    BaseClassDataType,
+    DataModel,
+    DataModelFieldBase,
+)
 from datamodel_code_generator.model.enum import Enum
 from datamodel_code_generator.parser import DefaultPutDict, LiteralType
 from datamodel_code_generator.reference import ModelResolver, Reference
@@ -114,9 +119,10 @@ def sort_data_models(
             unresolved_reference_model_names = [m.path for m in unresolved_references]
             for model in unresolved_references:
                 indexes = [
-                    unresolved_reference_model_names.index(b.path)
+                    unresolved_reference_model_names.index(b.reference.path)
                     for b in model.base_classes
-                    if b.path in unresolved_reference_model_names
+                    if b.reference
+                    and b.reference.path in unresolved_reference_model_names
                 ]
                 if indexes:
                     ordered_models.append((min(indexes), model,))
@@ -326,6 +332,8 @@ class Parser(ABC):
             aliases=aliases,
             empty_field_name=empty_enum_field_name,
             snake_case_field=snake_case_field,
+            custom_class_name_generator=custom_class_name_generator,
+            base_path=self.base_path,
         )
         self.class_name: Optional[str] = class_name
 
@@ -411,14 +419,36 @@ class Parser(ABC):
             (k, [*v]) for k, v in grouped_models
         ):  # type: Tuple[str, ...], List[DataModel]
 
-            # Remove duplicated name model
-            unique_model_names: Set[str] = set()
             for model in models:
-                if model.name in unique_model_names:
-                    # Remove duplicated name model
-                    models.remove(model)
-                    continue
-                unique_model_names.add(model.name)
+                if isinstance(model, self.data_model_root_type):
+                    root_data_type = model.fields[0].data_type
+
+                    # backward compatible
+                    # Remove duplicated root model
+                    if (
+                        root_data_type.reference
+                        and not root_data_type.is_dict
+                        and not root_data_type.is_list
+                        and root_data_type.reference.source in models
+                        and root_data_type.reference.name
+                        == self.model_resolver.get_class_name(
+                            model.reference.original_name, unique=False
+                        )
+                    ):
+                        # Replace referenced duplicate model to original model
+                        for child in model.reference.children[:]:
+                            child.replace_reference(root_data_type.reference)
+                        models.remove(model)
+                        continue
+
+                    #  Custom root model can't be inherited on restriction of Pydantic
+                    for child in model.reference.children:
+                        # inheritance model
+                        if isinstance(child, DataModel):
+                            for base_class in child.base_classes:
+                                if base_class.reference == model.reference:
+                                    child.base_classes.remove(base_class)
+
             module_models.append((module, models,))
 
             scoped_model_resolver = ModelResolver(
@@ -468,9 +498,16 @@ class Parser(ABC):
                         # Or, Referenced model is in the same file. we don't need to import the model
                         continue
 
-                    from_, import_ = full_path = relative(
-                        model.module_name, data_type.full_name
-                    )
+                    if isinstance(data_type, BaseClassDataType):
+                        from_ = ''.join(
+                            relative(model.module_name, data_type.full_name)
+                        )
+                        import_ = data_type.reference.short_name
+                        full_path = from_, import_
+                    else:
+                        from_, import_ = full_path = relative(
+                            model.module_name, data_type.full_name
+                        )
 
                     alias = scoped_model_resolver.add(full_path, import_).name
 
@@ -496,14 +533,13 @@ class Parser(ABC):
                     cached_model_reference = model_cache.get(model_key)
                     if cached_model_reference:
                         if isinstance(model, Enum):
-                            for child in model.reference.children:
-                                if isinstance(child, DataType):  # pragma: no cover
-                                    # child is resolved data_type by reference
-                                    data_model = get_most_of_parent(child)
+                            for child in model.reference.children[:]:
+                                # child is resolved data_type by reference
+                                data_model = get_most_of_parent(child)
 
-                                    # TODO: replace reference in all modules
-                                    if data_model in models:  # pragma: no cover
-                                        child.replace_reference(cached_model_reference)
+                                # TODO: replace reference in all modules
+                                if data_model in models:  # pragma: no cover
+                                    child.replace_reference(cached_model_reference)
                         else:
                             index = models.index(model)
                             inherited_model = model.__class__(
