@@ -1,7 +1,7 @@
 from enum import Enum
-from typing import List, Optional
+from typing import List, Optional, Union
 
-from pydantic import BaseModel
+from pydantic import BaseModel, Field
 
 
 class Kind(Enum):
@@ -74,3 +74,30 @@ class MultipleTypeEnum(Enum):
 
 class SingleEnum(Enum):
     pet = 'pet'
+
+
+class ArrayEnumEnum(Enum):
+    cat = 'cat'
+
+
+class ArrayEnumEnum1(Enum):
+    dog = 'dog'
+
+
+class ArrayEnum(BaseModel):
+    __root__: List[Union['ArrayEnumEnum', 'ArrayEnumEnum1']]
+
+
+class VersionEnum(Enum):
+    RC1 = 'RC1'
+    RC1N = 'RC1N'
+    RC2 = 'RC2'
+    RC2N = 'RC2N'
+    RC3 = 'RC3'
+    RC4 = 'RC4'
+
+
+class Version(BaseModel):
+    __root__: Optional['VersionEnum'] = Field(
+        'RC1', description='nullable enum', example='RC2'
+    )
