@@ -1,5 +1,5 @@
-from datamodel_code_generator import DataTypeManager
 from datamodel_code_generator.model import DataModelFieldBase
+from datamodel_code_generator.model.pydantic import DataTypeManager
 from datamodel_code_generator.model.pydantic.custom_root_type import CustomRootType
 from datamodel_code_generator.reference import Reference
 from datamodel_code_generator.types import DataType, Types
