@@ -1098,7 +1098,7 @@ def test_main_invalid_model_name():
 
 
 @freeze_time('2019-07-26')
-def test_main_root_id_jsonschema(mocker):
+def test_main_root_id_jsonschema_with_local_file(mocker):
     root_id_response = mocker.Mock()
     root_id_response.text = 'dummy'
     person_response = mocker.Mock()
@@ -1121,6 +1121,37 @@ def test_main_root_id_jsonschema(mocker):
             output_file.read_text()
             == (EXPECTED_MAIN_PATH / 'main_root_id' / 'output.py').read_text()
         )
+        httpx_get_mock.assert_not_called()
+    with pytest.raises(SystemExit):
+        main()
+
+
+@freeze_time('2019-07-26')
+def test_main_root_id_jsonschema_with_remote_file(mocker):
+    root_id_response = mocker.Mock()
+    root_id_response.text = 'dummy'
+    person_response = mocker.Mock()
+    person_response.text = (JSON_SCHEMA_DATA_PATH / 'person.json').read_text()
+    httpx_get_mock = mocker.patch('httpx.get', side_effect=[person_response])
+    with TemporaryDirectory() as output_dir:
+        input_file = Path(output_dir, 'root_id.json')
+        shutil.copy(JSON_SCHEMA_DATA_PATH / 'root_id.json', input_file)
+        output_file: Path = Path(output_dir) / 'output.py'
+        return_code: Exit = main(
+            [
+                '--input',
+                str(input_file),
+                '--output',
+                str(output_file),
+                '--input-file-type',
+                'jsonschema',
+            ]
+        )
+        assert return_code == Exit.OK
+        assert (
+            output_file.read_text()
+            == (EXPECTED_MAIN_PATH / 'main_root_id' / 'output.py').read_text()
+        )
         httpx_get_mock.assert_has_calls(
             [call('https://example.com/person.json'),]
         )
@@ -1129,7 +1160,7 @@ def test_main_root_id_jsonschema(mocker):
 
 
 @freeze_time('2019-07-26')
-def test_main_root_id_jsonschema_self_refs(mocker):
+def test_main_root_id_jsonschema_self_refs_with_local_file(mocker):
     person_response = mocker.Mock()
     person_response.text = (JSON_SCHEMA_DATA_PATH / 'person.json').read_text()
     httpx_get_mock = mocker.patch('httpx.get', side_effect=[person_response])
@@ -1151,6 +1182,36 @@ def test_main_root_id_jsonschema_self_refs(mocker):
         ).read_text().replace(
             'filename:  root_id.json', 'filename:  root_id_self_ref.json'
         )
+        httpx_get_mock.assert_not_called()
+    with pytest.raises(SystemExit):
+        main()
+
+
+@freeze_time('2019-07-26')
+def test_main_root_id_jsonschema_self_refs_with_remote_file(mocker):
+    person_response = mocker.Mock()
+    person_response.text = (JSON_SCHEMA_DATA_PATH / 'person.json').read_text()
+    httpx_get_mock = mocker.patch('httpx.get', side_effect=[person_response])
+    with TemporaryDirectory() as output_dir:
+        input_file = Path(output_dir, 'root_id_self_ref.json')
+        shutil.copy(JSON_SCHEMA_DATA_PATH / 'root_id_self_ref.json', input_file)
+        output_file: Path = Path(output_dir) / 'output.py'
+        return_code: Exit = main(
+            [
+                '--input',
+                str(input_file),
+                '--output',
+                str(output_file),
+                '--input-file-type',
+                'jsonschema',
+            ]
+        )
+        assert return_code == Exit.OK
+        assert output_file.read_text() == (
+            EXPECTED_MAIN_PATH / 'main_root_id' / 'output.py'
+        ).read_text().replace(
+            'filename:  root_id.json', 'filename:  root_id_self_ref.json'
+        )
         httpx_get_mock.assert_has_calls(
             [call('https://example.com/person.json'),]
         )
@@ -2023,6 +2084,54 @@ def test_main_all_of_ref():
         main()
 
 
+@freeze_time('2019-07-26')
+def test_main_all_of_with_object():
+    with TemporaryDirectory() as output_dir:
+        output_file: Path = Path(output_dir) / 'output.py'
+        with chdir(JSON_SCHEMA_DATA_PATH):
+            return_code: Exit = main(
+                [
+                    '--input',
+                    'all_of_with_object.json',
+                    '--output',
+                    str(output_file),
+                    '--input-file-type',
+                    'jsonschema',
+                ]
+            )
+            assert return_code == Exit.OK
+            assert (
+                output_file.read_text()
+                == (EXPECTED_MAIN_PATH / 'all_of_with_object' / 'output.py').read_text()
+            )
+    with pytest.raises(SystemExit):
+        main()
+
+
+@freeze_time('2019-07-26')
+def test_main_combined_array():
+    with TemporaryDirectory() as output_dir:
+        output_file: Path = Path(output_dir) / 'output.py'
+        with chdir(JSON_SCHEMA_DATA_PATH):
+            return_code: Exit = main(
+                [
+                    '--input',
+                    'combined_array.json',
+                    '--output',
+                    str(output_file),
+                    '--input-file-type',
+                    'jsonschema',
+                ]
+            )
+            assert return_code == Exit.OK
+            assert (
+                output_file.read_text()
+                == (EXPECTED_MAIN_PATH / 'combined_array' / 'output.py').read_text()
+            )
+    with pytest.raises(SystemExit):
+        main()
+
+
 @freeze_time('2019-07-26')
 def test_main_openapi_nullable():
     with TemporaryDirectory() as output_dir:
@@ -2178,6 +2287,67 @@ def test_main_generate_custom_class_name_generator():
         ).read_text().replace('Person', 'CustomPerson')
 
 
+@freeze_time('2019-07-26')
+def test_main_generate_custom_class_name_generator_modular(
+    tmpdir_factory: TempdirFactory,
+):
+    output_directory = Path(tmpdir_factory.mktemp('output'))
+
+    output_path = output_directory / 'model'
+    main_modular_custom_class_name_dir = (
+        EXPECTED_MAIN_PATH / 'main_modular_custom_class_name'
+    )
+
+    custom_class_name_generator = lambda name: f'Custom{name[0].upper() + name[1:]}'
+
+    with freeze_time(TIMESTAMP):
+        input_ = (OPEN_API_DATA_PATH / 'modular.yaml').relative_to(Path.cwd())
+        assert not input_.is_absolute()
+        generate(
+            input_=input_,
+            input_file_type=InputFileType.OpenAPI,
+            output=output_path,
+            custom_class_name_generator=custom_class_name_generator,
+        )
+
+        for path in main_modular_custom_class_name_dir.rglob('*.py'):
+            result = output_path.joinpath(
+                path.relative_to(main_modular_custom_class_name_dir)
+            ).read_text()
+            assert result == path.read_text()
+
+
+@freeze_time('2019-07-26')
+def test_main_generate_custom_class_name_generator_additional_properties(
+    tmpdir_factory: TempdirFactory,
+):
+    output_directory = Path(tmpdir_factory.mktemp('output'))
+
+    output_file = output_directory / 'models.py'
+
+    custom_class_name_generator = lambda name: f'Custom{name[0].upper() + name[1:]}'
+
+    input_ = (
+        JSON_SCHEMA_DATA_PATH / 'root_model_with_additional_properties.json'
+    ).relative_to(Path.cwd())
+    assert not input_.is_absolute()
+    generate(
+        input_=input_,
+        input_file_type=InputFileType.JsonSchema,
+        output=output_file,
+        custom_class_name_generator=custom_class_name_generator,
+    )
+
+    assert (
+        output_file.read_text()
+        == (
+            EXPECTED_MAIN_PATH
+            / 'main_root_model_with_additional_properties_custom_class_name'
+            / 'output.py'
+        ).read_text()
+    )
+
+
 @freeze_time('2019-07-26')
 def test_main_http_jsonschema(mocker):
     external_directory = JSON_SCHEMA_DATA_PATH / 'external_files_in_directory'
@@ -2195,9 +2365,9 @@ def test_main_http_jsonschema(mocker):
             get_mock_response('definitions/fur.json'),
             get_mock_response('definitions/friends.json'),
             get_mock_response('definitions/food.json'),
+            get_mock_response('definitions/machine/robot.json'),
             get_mock_response('definitions/drink/coffee.json'),
             get_mock_response('definitions/drink/tea.json'),
-            get_mock_response('person.json'),
         ],
     )
     with TemporaryDirectory() as output_dir:
@@ -2234,6 +2404,9 @@ def test_main_http_jsonschema(mocker):
                 call(
                     'https://example.com/external_files_in_directory/definitions/food.json'
                 ),
+                call(
+                    'https://example.com/external_files_in_directory/definitions/machine/robot.json'
+                ),
                 call(
                     'https://example.com/external_files_in_directory/definitions/drink/coffee.json'
                 ),
@@ -2500,3 +2673,107 @@ def test_main_jsonschema_special_field_name():
         )
     with pytest.raises(SystemExit):
         main()
+
+
+@freeze_time('2019-07-26')
+def test_main_jsonschema_complex_one_of():
+    with TemporaryDirectory() as output_dir:
+        output_file: Path = Path(output_dir) / 'output.py'
+        return_code: Exit = main(
+            [
+                '--input',
+                str(JSON_SCHEMA_DATA_PATH / 'complex_one_of.json'),
+                '--output',
+                str(output_file),
+                '--input-file-type',
+                'jsonschema',
+            ]
+        )
+        assert return_code == Exit.OK
+        assert (
+            output_file.read_text()
+            == (
+                EXPECTED_MAIN_PATH / 'main_jsonschema_complex_one_of' / 'output.py'
+            ).read_text()
+        )
+    with pytest.raises(SystemExit):
+        main()
+
+
+@freeze_time('2019-07-26')
+def test_main_jsonschema_complex_any_of():
+    with TemporaryDirectory() as output_dir:
+        output_file: Path = Path(output_dir) / 'output.py'
+        return_code: Exit = main(
+            [
+                '--input',
+                str(JSON_SCHEMA_DATA_PATH / 'complex_any_of.json'),
+                '--output',
+                str(output_file),
+                '--input-file-type',
+                'jsonschema',
+            ]
+        )
+        assert return_code == Exit.OK
+        assert (
+            output_file.read_text()
+            == (
+                EXPECTED_MAIN_PATH / 'main_jsonschema_complex_any_of' / 'output.py'
+            ).read_text()
+        )
+    with pytest.raises(SystemExit):
+        main()
+
+
+@freeze_time('2019-07-26')
+def test_main_jsonschema_combine_one_of_object():
+    with TemporaryDirectory() as output_dir:
+        output_file: Path = Path(output_dir) / 'output.py'
+        return_code: Exit = main(
+            [
+                '--input',
+                str(JSON_SCHEMA_DATA_PATH / 'combine_one_of_object.json'),
+                '--output',
+                str(output_file),
+                '--input-file-type',
+                'jsonschema',
+            ]
+        )
+        assert return_code == Exit.OK
+        assert (
+            output_file.read_text()
+            == (
+                EXPECTED_MAIN_PATH
+                / 'main_jsonschema_combine_one_of_object'
+                / 'output.py'
+            ).read_text()
+        )
+    with pytest.raises(SystemExit):
+        main()
+
+
+@freeze_time('2019-07-26')
+def test_main_jsonschema_combine_any_of_object():
+    with TemporaryDirectory() as output_dir:
+        output_file: Path = Path(output_dir) / 'output.py'
+        return_code: Exit = main(
+            [
+                '--input',
+                str(JSON_SCHEMA_DATA_PATH / 'combine_any_of_object.json'),
+                '--output',
+                str(output_file),
+                '--input-file-type',
+                'jsonschema',
+            ]
+        )
+        assert return_code == Exit.OK
+        assert (
+            output_file.read_text()
+            == (
+                EXPECTED_MAIN_PATH
+                / 'main_jsonschema_combine_any_of_object'
+                / 'output.py'
+            ).read_text()
+        )
+    with pytest.raises(SystemExit):
+        main()
